@@ -16,12 +16,19 @@ class BaseData():
         return next(self.cycled_data)
 
     def get_unique_random(self):
-        ls   = [dict(s) for s in set(frozenset(d.items()) for d in self.data)]
-        return random.choice(ls)
+        unique_items   = self.get_unique_items()
+        return random.choice(unique_items)
+
+    def get_unique_items(self):
+        return [dict(s) for s in set(frozenset(d.items()) for d in self.data)]
 
     def get_unique_random_value(self):
         return next(iter(self.get_unique_random().values()))
 
+    def get_hashed_value(self, value):
+        unique_items   = self.get_unique_items()
+        return unique_items[hash(value if value is not None else '_') % len(unique_items)]
+
     def get_hundred(self):
         return self.data
 
