@@ -1,3 +1,4 @@
+import math
 from greatawesomeutils.config import IS_DOCKER
 from greatawesomeutils.config import RETRY_ATTEMPTS_HIGH
 from greatawesomeutils.enums import *
@@ -25,7 +26,7 @@ from .selenium import save_screenshot
 from .selenium import save_screenshot
 from .selenium import SAVE_SCREENSHOT_ERROR_FOLDER, SAVE_SCREENSHOT_SUCCESS_FOLDER
 from .selenium import SAVE_SCREENSHOT_ERROR_FOLDER, SAVE_SCREENSHOT_SUCCESS_FOLDER, save_html_logs
-from .selenium_driver import delete_cache
+from .selenium_driver import RANDOM, delete_cache
 from .selenium_driver import delete_corrupted_files
 from .selenium_driver import GOOGLE_DRIVER, UNDETECTED_DRIVER, create_driver_by_driver_type, delete_profile_path, MyChrome
 
@@ -91,7 +92,8 @@ class RetryException(Exception):
 
 
 class RerunException(Exception):
-    pass
+    def __init__(self, after = None ):
+      self.after = after
 
 
 NULL = "NULL"
@@ -102,20 +104,21 @@ def has_user_id():
 
 
 class BaseTask():
-    data = json.loads(os.environ['data'])
-    user = json.loads(os.environ['user'])
-    task_type = os.environ['task_type']
-    MODULE_NAME = os.environ['MODULE_NAME']
-
-    task_id = os.environ['task_id']
-    SKIP_IP_CHECK = get_boolean_variable('SKIP_IP_CHECK')
+    data = json.loads(os.environ.get('data')) if os.environ.get('data') is not None else {} 
+    user =  json.loads(os.environ.get('user')) if os.environ.get('user') is not None else {}
+    task_type = os.environ.get('task_type')
+    MODULE_NAME = os.environ.get('MODULE_NAME', '')
 
+    task_id = os.environ.get('task_id')
+    SKIP_IP_CHECK = get_boolean_variable('SKIP_IP_CHECK') if os.environ.get('SKIP_IP_CHECK') is not None else True
+     
+    
     MASTER_API = MASTER_API_URL
 
     def run(self, driver: MyChrome, data, user):
-
         pass
 
+    
     def sign_up_success(self):
         r = requests.post(f"{BaseTask.MASTER_API}/tasks/sign-up-success/",
                           json={'task_id': BaseTask.task_id})
@@ -146,6 +149,8 @@ class BaseTask():
         return True
     is_eager = False
     start_link = None
+    
+    register_ip = True
 
     def begin_task(self):
         # Always use undetected in docker
@@ -155,10 +160,13 @@ class BaseTask():
             print(f'Skipping IP Check')
         else:
             ip = wait_till_ip_change()
-            is_success = BaseTask.register_ip_for_task(ip)
-            if not is_success:
-                ask_master_to_restart()
-            # Skip IP check next time
+            
+            if self.register_ip:
+                is_success = BaseTask.register_ip_for_task(ip)
+                
+                if not is_success:
+                    ask_master_to_restart()
+                # Skip IP check next time
             BaseTask.SKIP_IP_CHECK = True
 
         if not has_user_id():
@@ -192,8 +200,6 @@ class BaseTask():
 
             except Exception as error:
                 isReRun = isinstance(error, RerunException)
-                if isReRun:
-                    print('Bot is ReRunning')
                 
 
                 def print_driver_url():
@@ -207,30 +213,42 @@ class BaseTask():
                     traceback.print_exc()
                     print(error)
                     print_driver_url()
-                    save_html_logs(driver, user["user_id"])
-                    save_screenshot(driver, user["user_id"],
+                    save_html_logs(driver, user.get( "user_id", 'NULL'))
+                    save_screenshot(driver, user.get( "user_id", 'NULL'),
                                     SAVE_SCREENSHOT_ERROR_FOLDER, 'error')
 
                 BaseTask.close(driver)
 
                 if isinstance(error, BotCaughtException):
                     print('Deletig Profile')
-                    delete_profile_path(BaseTask.user["user_id"])
+                    if user.get( "user_id") is not None:
+                        delete_profile_path(BaseTask.user["user_id"])
                     ask_master_to_restart()
                 elif isinstance(error, EmailAlreadyExists):
                     print('Deletig Profile')
-                    delete_profile_path(BaseTask.user["user_id"])
+                    if user.get( "user_id") is not None:
+                        delete_profile_path(BaseTask.user["user_id"])
                     BaseTask.change_email()
                     ask_master_to_restart()
 
                 if isinstance(error, RetryException):
                     ask_master_to_restart()
                 if isReRun:
-                    do_run()
+                    if self.retries < self.max_retries:
+                        self.retries+=1
+                        if error.after is not None:
+                            sleep_for_n_seconds(error.after)
+
+                        print('Bot is ReRunning')
+                        do_run()
+                    
                 else:
                     exit_with_failed_status()
         do_run()
 
+    retries = 0
+    max_retries = math.inf  
+    
     def change_email():
         r = requests.patch(f"{BaseTask.MASTER_API}/tasks/change-email/",
                            json={'task_id': BaseTask.task_id})
@@ -239,6 +257,10 @@ class BaseTask():
 
     DRIVER_TYPE = GOOGLE_DRIVER
 
+
+    def has_user_id(self):
+        return  BaseTask.user.get("user_id") != NULL and BaseTask.user.get("user_id") is not None
+    
     def create_driver(self, is_eager):
 
         def do_create_driver():
@@ -247,13 +269,13 @@ class BaseTask():
                 driver_type = self.DRIVER_TYPE
                 return create_driver_by_driver_type(
                     driver_type,
-                    BaseTask.user["user_id"] if BaseTask.user["user_id"] != NULL else None,
-                    BaseTask.user["window_size"],
-                    BaseTask.user["user_agent"],
+                    BaseTask.user.get("user_id") if self.has_user_id() else None,
+                    BaseTask.user.get("window_size", RANDOM),
+                    BaseTask.user.get("user_agent", RANDOM),
                     is_eager)
 
             driver = retry_if_is_error(
-                run, NETWORK_ERRORS + [(WebDriverException, lambda: delete_corrupted_files(BaseTask.user["user_id"]))], RETRY_ATTEMPTS_HIGH)
+                run, NETWORK_ERRORS + [(WebDriverException, lambda: delete_corrupted_files(BaseTask.user["user_id"]) if self.has_user_id() else None )], RETRY_ATTEMPTS_HIGH)
 
             def call_quit(signalNumber, frame):
                 if is_exiting():
@@ -299,13 +321,6 @@ class BaseTask():
         return result
 
 
-def short_random_sleep():
-    sleep_for_n_seconds(random.randint(2, 4))
-
-
-def long_random_sleep():
-    sleep_for_n_seconds(random.randint(7, 17))
-
 
 def sleep_for_word_length_type(s):
 
