@@ -5,6 +5,8 @@ import pytz
 import errno
 import json
 import sys
+import random
+import pandas as pd
 from time import sleep
 import requests
 from requests.exceptions import ReadTimeout
@@ -164,7 +166,9 @@ def write_json(path, data, indent=4):
 
 
 def write_temp_json(data, indent=4):
-    with open('./temp.json', 'w') as fp:
+    path = './temp.json'
+    print('Writing ./temp.json')
+    with open(path, 'w') as fp:
         json.dump(data, fp, indent=indent)
 
 
@@ -374,6 +378,32 @@ def partition_by_percentage(percentage, iterable):
     return iterable[0:split_point], iterable[split_point:N]
 
 
+def safe_pop(ls, index = 0):
+        try:
+          return ls.pop(index)
+        except:
+          return None
+
+def safe_index(ls, index = 0):
+        try:
+          return ls[index]
+        except:
+          return None
+
+
+def short_random_sleep():
+    sleep_for_n_seconds(random.uniform(2, 4))
+
+def long_random_sleep():
+    sleep_for_n_seconds(random.uniform(7, 17))
+
+
+def write_csv(data):
+    df = pd.DataFrame(data)
+    path = './result.csv'
+    print(f'Writing {path}')
+    df.to_csv(path, encoding='utf-8', index=False)
+
 if __name__ == '__main__':
     # print(get_utc_time())
     # print(datetime.now())
