@@ -1,4 +1,4 @@
-from .selenium import get_element_by_id, get_element_or_none_by_name, get_element_or_none_by_selector, get_element_or_none_by_text, get_element_or_none_by_text_contains, get_elements_or_none_by_selector, get_html, js_click, print_body
+from .selenium import *
 from greatawesomeutils.config import IS_DOCKER
 from greatawesomeutils.config import PROFILES_PATH
 from greatawesomeutils.lang import silentremove
@@ -25,7 +25,7 @@ class DriverUtils():
         # Wait for dom to be interactable
         sleep(2)
 
-    def get_by_script(self, link, wait=2):
+    def get_by_current_page_referrer(self, link, wait=2):
         self.execute_script(f"""
                 window.location.href = "{link}";
             """)
@@ -35,9 +35,39 @@ class DriverUtils():
     def js_click(self, element):
         return js_click(self, element)
 
+
     def get_element_or_none_by_selector(self, selector, wait=None):
         return get_element_or_none_by_selector(self, selector, wait)
 
+#     await new Promise((resolve) => {
+#       var totalHeight = 0;
+#       var distance = 100;
+#       let count = 0;
+
+#       var timer = setInterval(() => {
+#         count++  
+#         var scrollHeight = document.body.scrollHeight;
+#           window.scrollBy(0, distance);
+#           totalHeight += distance;
+#           if (count >=5 ){
+#             clearInterval(timer);
+#             return resolve();
+
+#           }else{
+
+#           if(totalHeight >= scrollHeight - window.innerHeight){
+#               clearInterval(timer);
+#               resolve();
+#           }}
+#       }, 100);
+#   });
+
+    def scroll_some(self):
+        self.execute_script(""" 
+window.scrollBy(0, 10000);
+""")
+
+
     def get_element_by_id(self, selector, wait=None):
         return get_element_by_id(self, selector, wait)
 
@@ -74,8 +104,12 @@ class DriverUtils():
     def get_element_or_none_by_text(self, text, wait=None):
         return get_element_or_none_by_text(self, text, wait)
 
+
     def get_element_or_none_by_name(self, selector, wait=None):
         return get_element_or_none_by_name(self, selector, wait)
+   
+    def get_element_or_none(self, path, wait=None):
+        return get_element_or_none(self, path, wait)
 
     def get_html(self):
         return get_html(self)
@@ -119,13 +153,17 @@ class DriverUtils():
 
         def run(driver, link):
             visit(driver)
-            driver.get_by_script(link, wait)
+            driver.get_by_current_page_referrer(link, wait)
 
         run(self, link)
 
     def organic_get(self, link, wait=2):
         self.organic_get_via(link, "https://www.google.com/", wait)
 
+
+    def get_innerhtml(self, el):
+        return el.get_attribute("innerHTML")
+
     def get_cookies_as_dict(self):
         all_cookies = self.get_cookies()
         cookies_dict = {}
@@ -191,7 +229,7 @@ def add_useragent(options, user_agent):
 
 def create_profile_path(user_id):
     PATH = f'{PROFILES_PATH}/{user_id}'
-    path = relative_path(PATH, 1)
+    path = relative_path(PATH, 2)
     return path
 
 
@@ -213,6 +251,7 @@ def delete_profile_path(user_id):
 
 
 RANDOM = "RANDOM"
+HASHED = "HASHED"
 
 
 def add_essential_options(options, user_id, window_size, user_agent):
@@ -221,6 +260,8 @@ def add_essential_options(options, user_id, window_size, user_agent):
     if window_size is not None:
         if window_size == RANDOM:
             window_size = WindowSizesInstance.get_unique_random_value()
+        elif window_size == HASHED:
+            window_size = WindowSizesInstance.get_hashed_value(user_agent)
 
         print("window-size: ", window_size)
         options.add_argument(f"--window-size={window_size}")
@@ -228,6 +269,8 @@ def add_essential_options(options, user_id, window_size, user_agent):
     if user_agent is not None:
         if user_agent == RANDOM:
             user_agent = UserAgentsInstance.get_unique_random_value()
+        elif user_agent == HASHED:
+            user_agent = UserAgentsInstance.get_hashed_value(user_agent)
 
         print("user_agent: ", user_agent)
         add_useragent(options, user_agent)
@@ -259,11 +302,18 @@ GOOGLE_DRIVER = 'google_driver'
 
 def hide_automation_flags(options):
     options.add_argument('--disable-blink-features=AutomationControlled')
+    options.add_argument("--disable-blink-features")
 
     options.add_experimental_option(
         "excludeSwitches", ["enable-automation"])
     options.add_experimental_option('useAutomationExtension', False)
 
+    # New Options
+    options.add_argument("--ignore-certificate-errors")
+    options.add_argument('--no-sandbox')
+    options.add_argument("--disable-extensions")
+
+
 
 def create_driver_by_driver_type(driver_type, user_id, window_size, user_agent, is_eager):
 
@@ -307,11 +357,14 @@ def create_driver_by_driver_type(driver_type, user_id, window_size, user_agent,
         else:
             desired_capabilities = None
         print("Creating Driver")
-        hide_automation_flags(options)
+        
+
         if is_undetected:
             driver = MyChrome(desired_capabilities=desired_capabilities,
-                              options=options, version_main=108)
+                              options=options, version_main= 106 if IS_DOCKER else 108)
         else:
+            options.add_experimental_option("prefs",  {"profile.managed_default_content_settings.images": 2})
+            hide_automation_flags(options)
 
             
             # CAPTCHA
@@ -319,6 +372,7 @@ def create_driver_by_driver_type(driver_type, user_id, window_size, user_agent,
                 ["--disable-web-security", "--disable-site-isolation-trials", "--disable-application-cache"])
 
             path = relative_path("drivers/chromedriver.exe", 1)
+            print(f'driver path: {path}' )
             driver = MyGoogleChrome(
                 desired_capabilities=desired_capabilities,
                 chrome_options=options,
